#include <string.h>
#include <dirent.h>
#include "config.h"
#include "cdbslib.h"
#include "loadfile.h"

//===================================================================
// FILESYSTEM LIBRARY
//   Name: wild_expand
//===================================================================

/* WILD_EXPAND -- Expand an pattern containing filename wildcards
 *
 * Open a directory and check all files in it against a pattern
 *
 * i: pattern	wildcard pattern to be expanded
 * i: maxlist	maximum number if expanded pattern
 * u: ilist	next slot in list of expanded patterns
 * o: list	list of expanded patterns
 */

void wild_expand (const char *pattern, const int maxlist, 
		  int *ilist, char ** list)
{
    DIR			*fd;
    struct dirent	*entry;
    char		*name;

    int		olist = *ilist;
    char	dir[SZ_FNAME+1], root[SZ_FNAME+1], ext[SZ_FNAME+1];

    /* No need for expansion if no wild cards in pattern */

    if (strchr (pattern, '?') == NULL && strchr (pattern, '*') == NULL && 
	*ilist < maxlist) {

	list[(*ilist)++] = strclone (pattern);
	return;
    }

    breakfname (pattern, dir, root, ext, SZ_FNAME);
    strcat (root, ext);

    /* Try to match all files in directory with the pattern */

    if ((fd = opendir (dir)) != NULL) {
	while ((entry  = readdir (fd)) != NULL) {
	    name = entry->d_name;
	    if (patmatch (root, name))
		list [(*ilist)++] = strclone (name);
	}

	closedir (fd);
    }

    /* Copy pattern if no expansions performed */

    if (*ilist == olist && *ilist < maxlist)
	list[(*ilist)++] = strclone (pattern);
}
